/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.common.swaggercombine;

import com.google.gson.JsonObject;
import org.junit.Test;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertTrue;

public class SwaggerCombineTest {

    private static final String API1_YAML_FILE = "/systems-swagger.yaml";
    private static final String API2_YAML_FILE = "/jvms-swagger.yaml";

    @Test
    public void testCombine2() throws IOException {
        final File yamlFile1 = new File(getClass().getResource(API1_YAML_FILE).getFile());
        final File yamlFile2 = new File(getClass().getResource(API2_YAML_FILE).getFile());
        final File[] args = { yamlFile1, yamlFile2 };
        // combines two swagger definitions, and tests they actually have definitions for paths from each
        final SwaggerCombine sc = new SwaggerCombine();
        JsonObject combined = sc.processAPIs(toJsonList(args), true, false, false);
        assertTrue(combined.has("paths"));
        JsonObject paths = combined.getAsJsonObject("paths");
        assertTrue(paths.has("/jvms/0.0.1/systems/{systemId}"));
        assertTrue(paths.has("/systems/0.0.1/systems/{systemId}"));
    }

    @Test
    public void testDiff() throws IOException {
        final File yamlFile1 = new File(getClass().getResource(API1_YAML_FILE).getFile());
        final File yamlFile2 = new File(getClass().getResource(API2_YAML_FILE).getFile());
        final File[] args = { yamlFile1, yamlFile2 };
        // combines two swaggewr definitions, and tests the 'offset' key differs (in production it should always be optional)
        final StringBuffer buf = new StringBuffer();
        final SwaggerCombine sc = new SwaggerCombine() {
            protected void error(String msg) {
                buf.append(msg);
            }
        };
        sc.processAPIs(toJsonList(args), true, false, false);
        assertTrue(buf.toString().contains("key offset differs"));
    }

    private List<JsonObject> toJsonList(File[] files) throws IOException {
        List<JsonObject> list = new ArrayList<>(files.length);
        for (File fn : files) {
            JsonObject spec = SwaggerCombine.readSwaggerFile(fn);
            list.add(spec);
        }
        return list;
    }
}
